/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jdesigner.application.template.service;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpSession;

import org.jdesigner.platform.data.Tree;
import org.jdesigner.platform.exception.DatabaseException;

import org.jdesigner.application.template.domain.MenuLocation;

/**
 * <B>메뉴 서비스</B>
 * 
 * @author leesangboo
 * 
 */
public interface MenuService {

	//
	public static final String MENU_LOCATION_KEY = "org.jdesigner.application.template.service.menu.MENU_LOCATION";

	/**
	 * 전체 메뉴 트리 리스트를 반환한다.
	 * 
	 * @param params
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectAllMenuTreeList() throws CommonServiceException;

	/**
	 * 전체 메뉴 리스트를 반환한다.
	 * 
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectFullMenuTreeList() throws CommonServiceException;

	/**
	 * 사용자의 접근 가능한 메뉴 트리를 반환한다.
	 * 
	 * @return
	 * @throws CommonServiceException
	 */
	public Tree selectMenuTreeByUsername(Map params) throws CommonServiceException;

	/**
	 * 상단 네비게이션 메뉴 목록을 반환한다.
	 * 
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectPrimaryMenuListByUsername(HttpSession session) throws CommonServiceException;

	/**
	 * 왼쪽 서브 메뉴 목록을 반환한다. (레벨2)
	 * 
	 * @param parent_menu_code
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectSecondMenuListByUsername(HttpSession session, String parent_menu_code) throws CommonServiceException;

	/**
	 * 왼쪽 서브 메뉴 목록을 반환한다. (레벨3)
	 * 
	 * @param parent_menu_code
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectThirdMenuListByUsername(HttpSession session, String parent_menu_code) throws CommonServiceException;

	/**
	 * 선택된 메뉴가 세션이 존재 여부를 반환한다.
	 * 
	 * @param params
	 * @return
	 * @throws DatabaseException
	 */
	public boolean hasSelectedMenuLocationInSession(HttpSession session) throws CommonServiceException;

	/**
	 * 선택된 메뉴가 세션이 존재 여부를 반환한다.
	 * 
	 * @param params
	 * @return
	 * @throws DatabaseException
	 */
	public MenuLocation getSelectedMenuLocationInSession(HttpSession session) throws CommonServiceException;

	/**
	 * 메뉴 트리 리스트를 반환한다.
	 * 
	 * @param params
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectMenuTreeList(Map params) throws CommonServiceException;

	/**
	 * 메뉴 레벨 리스트를 반환한다.
	 * 
	 * @param params
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectMenuLevelList(Map params) throws CommonServiceException;

	/**
	 * 선택된 메뉴 역활 권한 리스트를 반환한다.
	 * 
	 * @return
	 * @throws Exception
	 */
	public List getMenuSelectedRuleList() throws CommonServiceException;

	/**
	 * URL과 매칭되는 메뉴를 반환한다.
	 * 
	 * @param url
	 * @return
	 * @throws CommonServiceException
	 */
	public Map selectMenuByURL(String url) throws CommonServiceException;

	/**
	 * URL과 매칭되는 메뉴 역환 권한을 반환한다.
	 * 
	 * @param url
	 * @return
	 * @throws Exception
	 */
	public MenuLocation getMenuLocationByURL(String url, String query) throws CommonServiceException;

	/**
	 * 상단 네비게이션 메뉴 목록을 반환한다.
	 * 
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectPrimaryMenuList() throws CommonServiceException;

	/**
	 * 왼쪽 서브 메뉴 목록을 반환한다. (레벨2)
	 * 
	 * @param parent_menu_code
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectSecondMenuList(String parent_menu_code) throws CommonServiceException;

	/**
	 * 왼쪽 서브 메뉴 목록을 반환한다. (레벨3)
	 * 
	 * @param parent_menu_code
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectThirdMenuList(String parent_menu_code) throws CommonServiceException;

	/**
	 * 메뉴를 조회한다.
	 * 
	 * @param params
	 * @return
	 * @throws CommonServiceException
	 */
	public Map selectMenu(Map params) throws CommonServiceException;

	/**
	 * 메뉴를 등록한다.
	 * 
	 * @param params
	 * @throws CommonServiceException
	 */
	public void insertMenu(Map params, List auth_role_list) throws CommonServiceException;

	/**
	 * 메뉴를 수정한다.
	 * 
	 * @param params
	 * @throws CommonServiceException
	 */
	public void updateMenu(Map params, List auth_role_list) throws CommonServiceException;

	/**
	 * 메뉴를 삭제한다.
	 * 
	 * @param params
	 * @throws CommonServiceException
	 */
	public void deleteMenu(Map params) throws CommonServiceException;

	/**
	 * 메뉴 트리 롤 권한 리스트를 반환한다.
	 * 
	 * @param params
	 * @return
	 * @throws CommonServiceException
	 */
	public List selectMenuTreeRoleList(Map params) throws CommonServiceException;

	/**
	 * 메뉴 트리 롤 권한을 반환한다.
	 * 
	 * @param params
	 * @return
	 * @throws CommonServiceException
	 */
	public Map selectMenuTreeRole(Map params) throws CommonServiceException;

	/**
	 * 메뉴 트리 롤 권한을 입력한다.
	 * 
	 * @param params
	 * @throws CommonServiceException
	 */
	public void insertMenuTreeRole(Map params) throws CommonServiceException;

	/**
	 * 메뉴 트리 롤 권한을 수정한다.
	 * 
	 * @param params
	 * @throws CommonServiceException
	 */
	public void updateMenuTreeRole(Map params) throws CommonServiceException;

	/**
	 * 메뉴 트리 롤 권한을 삭제한다.
	 * 
	 * @param params
	 * @throws CommonServiceException
	 */
	public void deleteMenuTreeRole(Map params) throws CommonServiceException;

	/**
	 * 캐쉬된 메뉴 팩토리를 초기화한다.
	 * 
	 * @throws CommonServiceException
	 */
	public void resetMenuSingletoneFactory() throws CommonServiceException;
}
