/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.application.template.service.provider;

import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.net.ftp.FTPFile;

import org.jdesigner.application.template.service.CommonServiceException;

/**
 * 
 * @author SangBoo Lee
 * @date 2009. 11. 16.
 * @since 1.0
 * 
 */
public interface FTP extends Cloneable {

	/***
	 * A constant used to indicate the file(s) being transfered should be treated as ASCII. This is the default file type. All constants ending in
	 * <code>FILE_TYPE</code> are used to indicate file types.
	 ***/
	public static final int ASCII_FILE_TYPE = 0;

	/***
	 * A constant used to indicate the file(s) being transfered should be treated as EBCDIC. Note however that there are several different EBCDIC
	 * formats. All constants ending in <code>FILE_TYPE</code> are used to indicate file types.
	 ***/
	public static final int EBCDIC_FILE_TYPE = 1;

	/***
	 * A constant used to indicate the file(s) being transfered should be treated as a binary image, i.e., no translations should be performed. All
	 * constants ending in <code>FILE_TYPE</code> are used to indicate file types.
	 ***/
	public static final int IMAGE_FILE_TYPE = 2;

	/***
	 * A constant used to indicate the file(s) being transfered should be treated as a binary image, i.e., no translations should be performed. All
	 * constants ending in <code>FILE_TYPE</code> are used to indicate file types.
	 ***/
	public static final int BINARY_FILE_TYPE = 2;

	/**
	 * 
	 * @return <code>true</code> if connected, <code>otherwise</code>
	 */
	public boolean isConnected();

	/**
	 * 
	 * @param encoding
	 * @throws CommonServiceException
	 */
	public void setControlEncoding(String encoding) throws CommonServiceException;

	/**
	 * Sets the file type to be transferred ( Check implementation to see the values )
	 * 
	 * @param arg0
	 *            the fileType
	 * @return <code>true</code> if successfull <code>false</code> otherwise
	 * @throws FtpServiceException
	 */
	public boolean setFileType(int arg0) throws CommonServiceException;

	/***************************************************************************
	 * Change the current working directory of the FTP session. <p>
	 * 
	 * @param pathname
	 *            The new current working directory.
	 * @return True if successfully completed, false if not.
	 * @exception CommonServiceException
	 *                if there is any error
	 **************************************************************************/
	public boolean changeWorkingDirectory(String pathname) throws CommonServiceException;

	/***************************************************************************
	 * Deletes a file on the FTP server. <p>
	 * 
	 * @param pathname
	 *            The pathname of the file to be deleted.
	 * @return True if successfully completed, false if not.
	 * @exception CommonServiceException
	 *                if there is any error
	 **************************************************************************/
	public boolean deleteFile(String pathname) throws CommonServiceException;

	/***************************************************************************
	 * Closes the connection to the FTP server and restores connection parameters to the default values. <p>
	 * 
	 * @exception CommonServiceException
	 *                If an error occurs while disconnecting.
	 **************************************************************************/
	public void disconnect() throws CommonServiceException;

	/**
	 * Using the default system autodetect mechanism, obtain a list of file information for the current working directory. <p> This information is
	 * obtained through the LIST command. The contents of the returned array is determined by the<code> FTPFileEntryParser </code> used. <p>
	 * 
	 * @return The list of file information contained in the current directory in the format determined by the autodetection mechanism. <p> <b>
	 *         NOTE:</b> This array may contain null members if any of the individual file listings failed to parse. The caller should check each
	 *         entry for null before referencing it.
	 * @exception CommonServiceException
	 *                if there is any error.
	 */
	public FTPFile[] listFiles() throws CommonServiceException;

	/***************************************************************************
	 * Obtain a list of filenames in the current working directory This information is obtained through the NLST command. If the current directory
	 * contains no files, a zero length array is returned only if the FTP server returned a positive completion code, otherwise, null is returned (the
	 * FTP server returned a 550 error No files found.). If the directory is not empty, an array of filenames in the directory is returned. <p>
	 * 
	 * @return The list of filenames contained in the current working directory. null if the list could not be obtained. If there are no filenames in
	 *         the directory, a zero-length array is returned.
	 * @exception CommonServiceException
	 *                if there is any problem.
	 **************************************************************************/
	public String[] listNames() throws CommonServiceException;

	/***************************************************************************
	 * Obtain a list of filenames in a directory (or just the name of a given file, which is not particularly useful). This information is obtained
	 * through the NLST command. If the given pathname is a directory and contains no files, a zero length array is returned only if the FTP server
	 * returned a positive completion code, otherwise null is returned (the FTP server returned a 550 error No files found.). If the directory is not
	 * empty, an array of filenames in the directory is returned. If the pathname corresponds to a file, only that file will be listed. The server may
	 * or may not expand glob expressions. <p>
	 * 
	 * @param pathname
	 *            The file or directory to list.
	 * @return The list of filenames contained in the given path. null if the list could not be obtained. If there are no filenames in the directory,
	 *         a zero-length array is returned.
	 * @exception CommonServiceException
	 *                If there is any error.
	 **************************************************************************/
	public String[] listNames(String pathname) throws CommonServiceException;

	/***************************************************************************
	 * Login to the FTP server using the provided username, password, and account. If no account is required by the server, only the username and
	 * password, the account information is not used. <p>
	 * 
	 * @param username
	 *            The username to login under.
	 * @param password
	 *            The password to use.
	 * @param account
	 *            The account to use.
	 * @return True if successfully completed, false if not.
	 * @exception CommonServiceException
	 *                If there is any error.
	 **************************************************************************/
	public boolean login(String username, String password, String account) throws CommonServiceException;

	/***************************************************************************
	 * Logout of the FTP server by sending the QUIT command. <p>
	 * 
	 * @return True if successfully completed, false if not.
	 * @exception CommonServiceException
	 *                If there is any error
	 **************************************************************************/
	public boolean logout() throws CommonServiceException;

	/***************************************************************************
	 * Creates a new subdirectory on the FTP server in the current directory (if a relative pathname is given) or where specified (if an absolute
	 * pathname is given). <p>
	 * 
	 * @param pathname
	 *            The pathname of the directory to create.
	 * @return True if successfully completed, false if not.
	 * @exception CommonServiceException
	 *                if there is any error.
	 **************************************************************************/
	public boolean makeDirectory(String pathname) throws CommonServiceException;

	/***************************************************************************
	 * Retrieves a named file from the server and writes it to the given OutputStream. This method does NOT close the given OutputStream. If the
	 * current file type is ASCII, line separators in the file are converted to the local representation. <p>
	 * 
	 * @param remote
	 *            The name of the remote file.
	 * @param local
	 *            The local OutputStream to which to write the file.
	 * @return True if successfully completed, false if not.
	 * @exception CommonServiceException
	 *                if there is any error
	 **************************************************************************/
	public boolean retrieveFile(String remote, OutputStream local) throws CommonServiceException;

	/***************************************************************************
	 * Returns an InputStream from which a named file from the server can be read. If the current file type is ASCII, the returned InputStream will
	 * convert line separators in the file to the local representation. You must close the InputStream when you finish reading from it. The
	 * InputStream itself will take care of closing the parent data connection socket upon being closed. To finalize the file transfer you must call
	 * completePendingCommand and check its return value to verify success. <p>
	 * 
	 * @param remote
	 *            The name of the remote file.
	 * @return An InputStream from which the remote file can be read. If the data connection cannot be opened (e.g., the file does not exist), null is
	 *         returned (in which case you may check the reply code to determine the exact reason for failure).
	 * @exception CommonServiceException
	 *                if there is any error.
	 **************************************************************************/
	public InputStream retrieveFileStream(String remote) throws CommonServiceException;

	/***************************************************************************
	 * Stores a file on the server using the given name and taking input from the given InputStream. This method does NOT close the given InputStream.
	 * If the current file type is ASCII, line separators in the file are transparently converted to the NETASCII format (i.e., you should not attempt
	 * to create a special InputStream to do this). <p>
	 * 
	 * @param remote
	 *            The name to give the remote file.
	 * @param local
	 *            The local InputStream from which to read the file.
	 * @return True if successfully completed, false if not.
	 * @exception CommonServiceException
	 *                if there is any error.
	 **************************************************************************/
	public boolean storeFile(String remote, InputStream local) throws CommonServiceException;

	/***************************************************************************
	 * Stores a file on the server using the given name and taking input from the given InputStream. This method does NOT close the given InputStream.
	 * If the current file type is ASCII, line separators in the file are transparently converted to the NETASCII format (i.e., you should not attempt
	 * to create a special InputStream to do this). <p>
	 * 
	 * @param remote
	 *            The name to give the remote file.
	 * @param local
	 *            The local InputStream from which to read the file.
	 * @return True if successfully completed, false if not.
	 * @exception CommonServiceException
	 *                if there is any error.
	 **************************************************************************/
	public boolean storeUniqueFile(String arg0, InputStream arg1) throws CommonServiceException;

	/***************************************************************************
	 * A convenience method to send the FTP PASV command to the server, receive the reply, and return the reply code. Remember, it's up to you to
	 * interpret the reply string containing the host/port information. <p>
	 * 
	 * @return The reply code received from the server.
	 * @exception CommonServiceException
	 *                if there is any error
	 **************************************************************************/
	public int pasv() throws CommonServiceException;

	/***************************************************************************
	 * A convenience method to send the FTP MKD command to the server, receive the reply, and return the reply code. Remember, it's up to you to
	 * interpret the reply string containing the host/port information. <p>
	 * 
	 * @return The reply code received from the server.
	 * @exception CommonServiceException
	 *                if there is any error
	 **************************************************************************/
	public int mkd(String pathname) throws CommonServiceException;

	/***************************************************************************
	 * Sends an FTP command with no arguments to the server, waits for a reply and returns the numerical response code. After invocation, for more
	 * detailed information, the actual reply text can be accessed by calling getReplyString or getReplyStrings. <p>
	 * 
	 * @param command
	 *            The text representation of the FTP command to send.
	 * @return The integer value of the FTP reply code returned by the server in response to the command.
	 * @exception CommonServiceException
	 *                if there is any error
	 **************************************************************************/
	public int sendCommand(String command) throws CommonServiceException;

	/***************************************************************************
	 * Sends an FTP command to the server, waits for a reply and returns the numerical response code. After invocation, for more detailed information,
	 * the actual reply text can be accessed by calling getReplyString or getReplyStrings. <p>
	 * 
	 * @param command
	 *            The FTPCommand constant corresponding to the FTP command to send.
	 * @param args
	 *            The arguments to the FTP command. If this parameter is set to null, then the command is sent with no argument.
	 * @return The integer value of the FTP reply code returned by the server in response to the command.
	 * @exception CommonServiceException
	 *                if there is any error
	 **************************************************************************/
	public int sendCommand(String command, String args) throws CommonServiceException;

	/**
	 * Method that will be used in order of creating new client instances
	 * 
	 * @return Object a new instance of the client
	 * @throws CloneNotSupportedException
	 */
	public Object clone() throws CloneNotSupportedException;

}
