/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.application.template.web.filter;

import java.io.IOException;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.async.AsynchronousExecutor;
import org.jdesigner.platform.async.Worker;
import org.jdesigner.platform.util.DateUtils;

import org.jdesigner.application.template.service.AnalysisService;
import org.jdesigner.application.template.service.CommonServiceFactory;
import org.jdesigner.application.template.service.factory.MonitoringLogBufferFactory;

import com.jamonapi.MonKeyImp;
import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
 * MonitoringFilter
 * 
 * @author SangBoo Lee
 * @date 2010. 2. 26.
 * @since 1.0
 * 
 */
public class MonitoringFilter extends HttpServlet implements Filter {

	private static final long serialVersionUID = 1L;

	private static final Log log = LogFactory.getLog(MonitoringFilter.class);

	private static final String MONITOR_KEY = "application.monitor.web.allPages";

	private static final String BUFFER_SIZE_KEY = "buffer";

	private static final String PRINT_LOG_KEY = "log";

	private static final int DEFUALT_BUFFER_SIZE = 100;

	private int bufferSize = DEFUALT_BUFFER_SIZE;

	private boolean printLog = false;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) throws ServletException {
		//
		if (config.getInitParameter(BUFFER_SIZE_KEY) != null) {
			bufferSize = Integer.parseInt(config.getInitParameter(BUFFER_SIZE_KEY));
		}
		//
		if (config.getInitParameter(PRINT_LOG_KEY) != null) {
			printLog = Boolean.parseBoolean(config.getInitParameter(PRINT_LOG_KEY));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.GenericServlet#destroy()
	 */
	public void destroy() {
		//
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain filterChain) throws IOException, ServletException {

		//
		Monitor allPages = MonitorFactory.start(new MonKeyImp(MONITOR_KEY, getURI(request), "ms."));
		Monitor monitor = MonitorFactory.start(getURI(request));

		//성능 모니터링
		try {
			filterChain.doFilter(request, response);
		} finally {
			monitor.stop();
			allPages.stop();
		}

		//모니터링 로그 버퍼 추가
		Map params = makeSQLParameterMap(monitor);
		MonitoringLogBufferFactory bufferFactory = MonitoringLogBufferFactory.getInstance();
		bufferFactory.addBuffer(params);

		//데이터베이스에 성능 모니터링 로그 저장
		if (bufferFactory.getBufferSize() == bufferSize) {
			//
			final List bufferList = bufferFactory.getBufferList();
			bufferFactory.resetBufferList();
			//
			AsynchronousExecutor executor = new AsynchronousExecutor();
			executor.execute(new Worker() {
				public void execute() {
					for (int i = 0; i < bufferList.size(); i++) {
						Map buffer = (Map) bufferList.get(i);
						AnalysisService service = CommonServiceFactory.getAnalysisService();
						service.insertMonitoringLog(buffer);
					}
				}
			});
		}

		//성능 모니터링 로그 프린트 여부
		if (printLog) {
			log.info(makeLogString(monitor));
		}

	}

	/**
	 * 
	 * @param request
	 * @return
	 */
	protected String getURI(ServletRequest request) {
		if (request instanceof HttpServletRequest) {
			return ((HttpServletRequest) request).getRequestURI();
		} else {
			return "Not an HttpServletRequest";
		}
	}

	/**
	 * 
	 * @param monitor
	 * @return
	 */
	protected Map makeSQLParameterMap(Monitor monitor) {
		if (monitor != null) {
			Map params = new HashMap();
			params.put("uri", monitor.getMonKey().getValue("Label"));
			params.put("last_value", monitor.getLastValue() + "");
			params.put("hits", monitor.getHits() + "");
			params.put("avg", monitor.getAvg() + "");
			params.put("total", monitor.getTotal() + "");
			params.put("min", monitor.getMin() + "");
			params.put("max", monitor.getMax() + "");
			params.put("active", monitor.getActive() + "");
			params.put("avg_active", monitor.getAvgActive() + "");
			params.put("max_active", monitor.getMaxActive() + "");
			params.put("first_access", new Timestamp(monitor.getFirstAccess().getTime()));
			params.put("last_access", new Timestamp(monitor.getLastAccess().getTime()));
			return params;
		} else {
			return null;
		}
	}

	/**
	 * 
	 * @param monitor
	 * @return
	 */
	protected String makeLogString(Monitor monitor) {
		if (monitor != null) {
			StringBuffer b = new StringBuffer(400);
			b.append("Performance monitoring ");
			b.append("for URI=[" + monitor.getMonKey().getValue("Label") + "] : (");
			b.append("LastValue=");
			b.append(monitor.getLastValue());
			b.append(", Hits=");
			b.append(monitor.getHits());
			b.append(", Avg=");
			b.append(monitor.getAvg());
			b.append(", Total=");
			b.append(monitor.getTotal());
			b.append(", Min=");
			b.append(monitor.getMin());
			b.append(", Max=");
			b.append(monitor.getMax());
			b.append(", Active=");
			b.append(monitor.getActive());
			b.append(", Avg Active=");
			b.append(monitor.getAvgActive());
			b.append(", Max Active=");
			b.append(monitor.getMaxActive());
			b.append(", First Access=");
			b.append(DateUtils.timestampToISO(new Timestamp(monitor.getFirstAccess().getTime()), true));
			b.append(", Last Access=");
			b.append(DateUtils.timestampToISO(new Timestamp(monitor.getLastAccess().getTime()), true));
			b.append(")");
			return b.toString();
		} else {
			return "";
		}
	}

}
