/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.jdesigner.platform.db;

import java.sql.Connection;
import java.util.List;
import java.util.Map;

import org.jdesigner.platform.db.query.NamedQuery;
import org.jdesigner.platform.db.query.Query;
import org.jdesigner.platform.db.query.TableQuery;
import org.jdesigner.platform.db.session.Session;
import org.jdesigner.platform.db.transaction.Transaction;

/**
 * <B>Persistence</B>
 * 
 * <P>Persistence 는 데이터베이스 연결 및 트랜잭션 생성, 쿼리의 실행을 처리합니다.</p>
 * 
 * 
 * @author leesangboo
 * 
 */
public interface Persistence {

	/**
	 * ID를 반환한다.
	 * 
	 * @return
	 */
	public String getId();

	/**
	 * 세션을 반환한다.
	 * 
	 * @return
	 */
	public Session getOpenedSession();

	/**
	 * 트랜잭션을 반환한다.
	 * 
	 * @return
	 */
	public Transaction getTransaction();

	/**
	 * 현재 JDBC 커넥션을 반환한다.
	 * 
	 * @return
	 */
	public Connection getConnection();

	/**
	 * 현재 JDBC 커넥션을 반환한다.
	 * 
	 * @return
	 */
	public void changeConnection(String datasourceJNDIName);

	/**
	 * 트랜잭션을 시작한다.
	 */
	public void startTransaction();

	/**
	 * 트랜잭션을 커밋한다.
	 */
	public void commitTransaction();

	/**
	 * 트랜잭션을 종료한다.
	 */
	public void endTransaction();

	/**
	 * Query를 생성한다.
	 * 
	 * @param path
	 * @return
	 * @deprecated <code>#newNamedQuery</code>를 사용하세요.
	 */
	public Query newQuery(String path);

	/**
	 * Query를 생성한다.
	 * 
	 * @param path
	 * @param parameters
	 * @return
	 * @deprecated <code>#newNamedQuery</code>를 사용하세요.
	 */
	public Query newQuery(String path, List parameters);

	/**
	 * NamedQuery를 생성한다.
	 * 
	 * @param path
	 * @return
	 */
	public NamedQuery newNamedQuery(String path);

	/**
	 * NamedQuery를 생성한다.
	 * 
	 * @param path
	 * @param parameters
	 * @return
	 */
	public NamedQuery newNamedQuery(String path, Map parameters);

	/**
	 * TableQuery를 생성한다.
	 * 
	 * @param tableName
	 * @return
	 */
	public TableQuery newTableQuery(String tableName);

	/**
	 * TableQuery를 생성한다.
	 * 
	 * @param tableName
	 * @param parameters
	 * @return
	 */
	public TableQuery newTableQuery(String tableName, Map parameters);

	/**
	 * TableQuery를 생성한다.
	 * 
	 * @param tableName
	 * @param parameters
	 * @param onlyParamKeyBinding
	 * @return
	 */
	public TableQuery newTableQuery(String tableName, Map parameters, boolean onlyParamKeyBinding);

	/**
	 * TableQuery를 생성한다.
	 * 
	 * @param tableName
	 * @param parameters
	 * @param processTargetColumns
	 * @return
	 */
	public TableQuery newTableQuery(String tableName, Map parameters, String[] processTargetColumns);

	/**
	 * TableQuery를 생성한다.
	 * 
	 * @param tableName
	 * @param primaryKeys
	 * @param parameters
	 * @return
	 */
	public TableQuery newTableQuery(String tableName, String[] primaryKeys, Map parameters);

	/**
	 * TableQuery를 생성한다.
	 * 
	 * @param tableName
	 * @param primaryKeys
	 * @param parameters
	 * @param onlyParamKeyBinding
	 * @return
	 */
	public TableQuery newTableQuery(String tableName, String[] primaryKeys, Map parameters, boolean onlyParamKeyBinding);

	/**
	 * TableQuery를 생성한다.
	 * 
	 * @param tableName
	 * @param primaryKeys
	 * @param parameters
	 * @param processTargetColumns
	 * @return
	 */
	public TableQuery newTableQuery(String tableName, String[] primaryKeys, Map parameters, String[] processTargetColumns);

}
