/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.processor.impl;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.data.Page;
import org.jdesigner.platform.db.domain.UpdateResult;
import org.jdesigner.platform.db.processor.QueryProcessingException;
import org.jdesigner.platform.db.processor.QueryValidateException;
import org.jdesigner.platform.db.processor.TableQueryProcessor;
import org.jdesigner.platform.db.query.TableQuery;

/**
 * TableQueryProcessorImpl
 * 
 * @author leesangboo
 * 
 */
public class TableQueryProcessorImpl extends SQLProcessor implements TableQueryProcessor {

	private static final Log log = LogFactory.getLog(TableQueryProcessorImpl.class);

	private TableQuery query;

	public TableQueryProcessorImpl(TableQuery query) {
		super();
		this.query = query;
		this.validate();
		log.info("Query : " + query.toString());
	}

	/**
	 * 필수 값 검증
	 * 
	 */
	private void validate() {
		if (query.getTableName() == null && query.getTableName().equals("")) {
			throw new QueryValidateException("TableQuery [tableName] value is null, this is required value.");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.TableQueryProcessor#findAll()
	 */
	public List selectAll() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, SELECT_ALL);
			Map parameters = query.getParameters();
			return super.selectList(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
		} catch (Exception ex) {
			String msg = "[findAll] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.processor.TableQueryProcessor#countAll()
	 */
	public long countAll() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, COUNT_ALL);
			Map parameters = query.getParameters();
			List result = super.selectList(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
			return (result != null) ? result.size() : 0;
		} catch (Exception ex) {
			String msg = "[countAll] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.TableQueryProcessor#findAllPage(int, int)
	 */
	public Page selectAllPage(int start, int range, String column, boolean desc) throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, SELECT_ALL);
			Map parameters = query.getParameters();
			return super.selectPage(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters, start, range, column, desc);
		} catch (Exception ex) {
			String msg = "[findAllPage] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.TableQueryProcessor#findByPrimaryKey()
	 */
	public Map selectByPrimaryKey() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, SELECT_BY_PRIMARY_KEY);
			Map parameters = query.getParameters();
			return super.selectMap(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
		} catch (Exception ex) {
			String msg = "[findByPrimaryKey] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	public long countByPrimaryKey() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, COUNT_BY_PRIMARY_KEY);
			Map parameters = query.getParameters();
			BigDecimal result = (BigDecimal) super.selectObject(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
			return (result != null) ? result.longValue() : 0;
		} catch (Exception ex) {
			String msg = "[countByPrimaryKey] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.TableQueryProcessor#findByCustomKey()
	 */
	public List selectByCustomKey() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, SELECT_BY_CUSTOM_KEY);
			Map parameters = query.getParameters();
			return super.selectList(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
		} catch (Exception ex) {
			String msg = "[findByCustomKey] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.processor.TableQueryProcessor#countByCustomKey()
	 */
	public long countByCustomKey() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, COUNT_BY_CUSTOM_KEY);
			Map parameters = query.getParameters();
			BigDecimal result = (BigDecimal) super.selectObject(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
			return (result != null) ? result.longValue() : 0;
		} catch (Exception ex) {
			String msg = "[countByCustomKey] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.TableQueryProcessor#insert()
	 */
	public UpdateResult insert() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, INSERT);
			Map parameters = query.getParameters();
			return super.update(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
		} catch (Exception ex) {
			String msg = "[insert] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.TableQueryProcessor#update()
	 */
	public UpdateResult update() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, UPDATE);
			Map parameters = query.getParameters();
			return super.update(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
		} catch (Exception ex) {
			String msg = "[update] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jdesigner.platform.db.TableQueryProcessor#delete()
	 */
	public UpdateResult delete() throws QueryProcessingException {
		try {
			String sql = super.getParsedDynamicSQL(query, DELETE);
			Map parameters = query.getParameters();
			return super.update(query.getSession().getConnection(), query.getSession().getTransaction().isActive(), sql, parameters);
		} catch (Exception ex) {
			String msg = "[delete] processing failed : " + ex.getMessage();
			log.error(msg, ex);
			throw new QueryProcessingException(msg, ex);
		}
	}

}
