/*
 *  JDesigner (R) Web Application Development Platform 
 *  Copyright (C) 2010 JDesigner / SangBoo Lee (leesangboo@gmail.com)
 * 
 *  http://www.jdesigner.org/
 *  
 *  Please contact if you require licensing terms other
 *  than GPL-3 for this software.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.jdesigner.platform.db.table;

import java.util.Arrays;
import java.util.Map;
import java.util.Properties;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jdesigner.platform.cache.Cache;
import org.jdesigner.platform.cache.CacheFactory;
import org.jdesigner.platform.cache.impl.LRUCache;
import org.jdesigner.platform.context.ApplicationConfigUtils;
import org.jdesigner.platform.db.parser.QueryParsingException;
import org.jdesigner.platform.db.query.TableQuery;
import org.jdesigner.platform.util.MapUtils;
import org.jdesigner.platform.util.StringUtils;

/**
 * TableSQLFactory
 * 
 * @author leesangboo
 * 
 */
public class TableSQLFactory {

	private static final Log log = LogFactory.getLog(TableSQLFactory.class);

	private static final String SQL_CACHE_KEY = "platform.db.sql.cache";
	private static final String CACHE_ID = "platform.db.sql.cache.sql.table";
	private static final int CACHE_SIZE = 1000;

	private Cache cache;

	private TableSQLLoader tableSqlLoader;

	private static class TableSQLFactoryHolder {
		static TableSQLFactory instance = new TableSQLFactory();
	}

	public static TableSQLFactory getInstance() {
		return TableSQLFactoryHolder.instance;
	}

	public TableSQLFactory() {
		//
		Properties config = new Properties();
		config.put(LRUCache.CONFIG_MAX_ENTRIES_KEY, String.valueOf(CACHE_SIZE));
		cache = CacheFactory.createCache(CACHE_ID, Cache.LRU, config);
		//
		tableSqlLoader = new TableSQLLoader();
	}

	/**
	 * SQL을 반환한다.
	 * 
	 * @param query
	 * @return
	 */
	public String getSQL(TableQuery query, String mode) throws QueryParsingException {
		boolean useCache = ApplicationConfigUtils.getBoolean(SQL_CACHE_KEY);
		String cacheKey = generateCacheKey(query, mode);
		if (useCache) { // 캐슁 사용 여부
			if (!cache.hasValue(cacheKey)) {
				String sql = tableSqlLoader.loadSQLFromTableQueryAndMode(query, mode);
				cache.put(cacheKey, sql);
				log.info("Table SQL cached : key=[" + cacheKey + "], cachedSize=[" + getSQLMap().size() + "]");
				return sql;
			} else {
				log.debug("Table SQL cached return : key=[" + cacheKey + "]");
				return (String) cache.get(cacheKey);
			}
		} else {
			return tableSqlLoader.loadSQLFromTableQueryAndMode(query, mode);
		}
	}

	/**
	 * 캐쉬 ID를 생성한다.
	 * 
	 * @param query
	 * @param mode
	 * @return
	 */
	private String generateCacheKey(TableQuery query, String mode) {
		StringBuffer generaterCacheKey = new StringBuffer();
		generaterCacheKey.append(query.getTableName()).append("_");
		generaterCacheKey.append(mode).append("_");
		generaterCacheKey.append(StringUtils.arrayToCommaDelimitedString(query.getPrimaryKeys())).append("_");
		Map keyMap = query.getParameters();
		if (keyMap != null) {
			SortedMap keyOrderMap = new TreeMap(keyMap);
			String bindColumnString = Arrays.toString(MapUtils.getKeyArray(keyOrderMap));
			generaterCacheKey.append(bindColumnString).append("_");
		}
		return generaterCacheKey.toString();
	}

	/**
	 * 캐쉬된 SQL 맵을 반환한다.
	 * 
	 * @return
	 */
	public Map getSQLMap() {
		return cache.toMap();
	}

}